@@ -28,6 +28,9 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdlib.h>
+#include <wctype.h>
+#include <wchar.h>
 
 #include <glib/garray.h>
 #include <glib/gmessages.h>
@@ -253,7 +256,7 @@ match_pattern (const char *scanner, const char **resulting_scanner, const char *
 }
 
 GnomeMagicEntry *
-gnome_vfs_mime_magic_parse (const gchar *filename, gint *nents)
+_gnome_vfs_mime_magic_parse (const gchar *filename, gint *nents)
 {
 	GArray *array;
 	GnomeMagicEntry newent, *retval;
@@ -528,7 +531,7 @@ gnome_vfs_mime_try_one_magic_pattern (GnomeVFSMimeSniffBuffer *sniff_buffer,
 
 	for (offset = magic_entry->range_start; offset <= magic_entry->range_end; offset++) {
 		/* this check is done only as an optimization
-		 * gnome_vfs_mime_sniff_buffer_get already implements the laziness.
+		 * _gnome_vfs_mime_sniff_buffer_get already implements the laziness.
 		 * This gets called a million times though and every bit performance
 		 * is valuable. This way we avoid making the call.
 		 */
@@ -536,7 +539,7 @@ gnome_vfs_mime_try_one_magic_pattern (GnomeVFSMimeSniffBuffer *sniff_buffer,
 		if (sniff_buffer->buffer_length < offset + magic_entry->pattern_length) {
 
 			if (!sniff_buffer->read_whole_file) {
-				if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, 
+				if (_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, 
 								     offset + magic_entry->pattern_length) != GNOME_VFS_OK) {
 					return FALSE;
 				}
@@ -564,7 +567,7 @@ gnome_vfs_mime_get_magic_table (void)
 	G_LOCK (mime_magic_table_mutex);
 
 	if (mime_magic_table == NULL) {
-		mime_magic_table = gnome_vfs_mime_magic_parse
+		mime_magic_table = _gnome_vfs_mime_magic_parse
 			(SYSCONFDIR "/gnome-vfs-mime-magic" , NULL);
 	}
 
@@ -574,7 +577,7 @@ gnome_vfs_mime_get_magic_table (void)
 }
 
 const char *
-gnome_vfs_mime_get_type_from_magic_table (GnomeVFSMimeSniffBuffer *buffer)
+_gnome_vfs_mime_get_type_from_magic_table (GnomeVFSMimeSniffBuffer *buffer)
 {
 	GnomeMagicEntry *magic_table;
 	
@@ -597,7 +600,7 @@ gnome_vfs_mime_test_get_magic_table (const char *table_path)
 {
 	G_LOCK (mime_magic_table_mutex);
   	if (mime_magic_table == NULL) {
-		mime_magic_table = gnome_vfs_mime_magic_parse (table_path, NULL);
+		mime_magic_table = _gnome_vfs_mime_magic_parse (table_path, NULL);
   	}
 	G_UNLOCK (mime_magic_table_mutex);
 
@@ -695,7 +698,7 @@ gnome_vfs_mime_dump_magic_table (void)
 }
 
 void
-gnome_vfs_mime_clear_magic_table (void)
+_gnome_vfs_mime_clear_magic_table (void)
 {
 	G_LOCK (mime_magic_table_mutex);
   	g_free (mime_magic_table);
@@ -715,26 +718,27 @@ gnome_vfs_mime_clear_magic_table (void)
 const char *
 gnome_vfs_get_mime_type_for_buffer (GnomeVFSMimeSniffBuffer *buffer)
 {
-	return gnome_vfs_get_mime_type_internal (buffer, NULL);
+	return _gnome_vfs_get_mime_type_internal (buffer, NULL);
 }
 
 enum {
 	GNOME_VFS_TEXT_SNIFF_LENGTH = 256
 };
 
+
 /**
- * gnome_vfs_sniff_buffer_looks_like_text:
+ * _gnome_vfs_sniff_buffer_looks_like_text:
  * @sniff_buffer: buffer to examine
  *
  * Return value: returns %TRUE if the contents of @sniff_buffer appear to
  * be text.
  **/
 gboolean
-gnome_vfs_sniff_buffer_looks_like_text (GnomeVFSMimeSniffBuffer *sniff_buffer)
+_gnome_vfs_sniff_buffer_looks_like_text (GnomeVFSMimeSniffBuffer *sniff_buffer)
 {
 	gchar *end;
 	
-	gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_TEXT_SNIFF_LENGTH);
+	_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_TEXT_SNIFF_LENGTH);
 
 	if (sniff_buffer->buffer_length == 0) {
 		return FALSE;
@@ -753,7 +757,51 @@ gnome_vfs_sniff_buffer_looks_like_text (GnomeVFSMimeSniffBuffer *sniff_buffer)
 
 		remaining_bytes -= (end-((gchar*)sniff_buffer->buffer));
 	
- 		return (g_utf8_get_char_validated(end, remaining_bytes) == -2);
+ 		if (g_utf8_get_char_validated(end, remaining_bytes) == -2)
+			return TRUE;
+		else {
+			size_t wlen;
+			wchar_t wc;
+			gchar *src, *end;
+			mbstate_t state;
+
+			src = sniff_buffer->buffer;
+			end = sniff_buffer->buffer + sniff_buffer->buffer_length;
+			
+			memset (&state, 0, sizeof (state));
+			while (src < end) {
+				/* Don't allow embedded zeros in textfiles */
+				if (*src == 0)
+					return FALSE;
+				
+				wlen = mbrtowc(&wc, src, end - src, &state);
+
+				if (wlen == (size_t)(-1)) {
+					/* Illegal mb sequence */
+					return FALSE;
+				}
+				
+				if (wlen == (size_t)(-2)) {
+					/* No complete mb char before end
+					 * Probably a cut off char which is ok */
+					return TRUE;
+				}
+
+				if (wlen == 0) {
+					/* Don't allow embedded zeros in textfiles */
+					return FALSE;
+				}
+				
+				if (!iswspace (wc)  && !iswprint(wc)) {
+					/* Not a printable or whitspace
+					 * Probably not a text file */
+					return FALSE;
+				}
+
+				src += wlen;
+			}
+			return TRUE;
+		}
 	} 
 }
 
@@ -819,19 +867,19 @@ enum {
 };
 
 /**
- * gnome_vfs_sniff_buffer_looks_like_mp3:
+ * _gnome_vfs_sniff_buffer_looks_like_mp3:
  * @sniff_buffer: buffer to examine
  *
  * Return value: returns %TRUE if the contents of @sniff_buffer appear to
  * be an MP3.
  **/
 gboolean
-gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
+_gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
 {
 	unsigned long mp3_header;
 	int offset;
 	
-	if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_MP3_SNIFF_LENGTH) != GNOME_VFS_OK) {
+	if (_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_MP3_SNIFF_LENGTH) != GNOME_VFS_OK) {
 		return FALSE;
 	}
 
@@ -855,6 +903,13 @@ gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
 		&& (sniff_buffer->buffer[7] < 0x80u)
 		&& (sniff_buffer->buffer[8] < 0x80u)
 		&& (sniff_buffer->buffer[9] < 0x80u)) {
+		/* checks for existance of vorbis identification header */
+		for (offset=10; offset < GNOME_VFS_MP3_SNIFF_LENGTH-7; offset++) {
+			if (strncmp ((char *) &sniff_buffer->buffer[offset], 
+				     "\x01vorbis", 7) == 0) {
+				return FALSE;
+			}
+		}
 		return TRUE;
 	}
 
@@ -880,7 +935,7 @@ gnome_vfs_sniff_buffer_looks_like_mp3 (GnomeVFSMimeSniffBuffer *sniff_buffer)
 			 */
 			offset += 1 + length;
 
-			if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, offset + 4) != GNOME_VFS_OK) {
+			if (_gnome_vfs_mime_sniff_buffer_get (sniff_buffer, offset + 4) != GNOME_VFS_OK) {
 				return FALSE;
 			}
 			mp3_header = get_4_byte_value (&sniff_buffer->buffer[offset]);
